@@ -2,7 +2,6 @@
     \brief Interface to the event handling of ASC
 */
 
-//     $Id: gameeventsystem.h,v 1.3 2005/04/02 13:57:06 mbickel Exp $
 /*
     This file is part of Advanced Strategic Command; http://www.asc-hq.de
     Copyright (C) 1994-1999  Martin Bickel  and  Marc Schellenberger
@@ -31,12 +30,11 @@
 
 #include "typen.h"
 #include "libs/loki/Singleton.h"
+#include "factory.h"
 
-#if defined(sgmain) || defined(karteneditor)
-# include "mapdisplay.h"
-#else
- class MapDisplayInterface {};
-#endif
+#include "mapdisplayinterface.h"
+
+class MapDisplayInterface;
 
 enum  EventConnections { cconnection_destroy = 1,
                          cconnection_conquer = 2,
@@ -46,17 +44,12 @@ enum  EventConnections { cconnection_destroy = 1,
                          cconnection_areaentered_specificunit = 32 };
 
 
-extern void  checkevents( MapDisplayInterface* md );
+extern bool  checkevents( MapDisplayInterface* md );
 
 extern void  checktimedevents( MapDisplayInterface* md );
 
 extern void eventReady();
 
-extern const int EventActionNum;
-extern const int EventTriggerNum;
-extern const char* EventTriggerName[];
-extern const char* EventActionName[];
-
 typedef int EventTriggerID;
 typedef int EventActionID;
 
@@ -69,7 +62,7 @@ class EventTrigger {
    protected:
       EventTrigger ( EventTriggerID id ) : triggerID ( id ), gamemap(NULL), event(NULL), stateCache(unfulfilled), triggerFinal( false ), invert(false) {};
       virtual State getState( int player ) = 0;
-      tmap* gamemap;
+      GameMap* gamemap;
       Event* event;
 
       bool isFulfilled();
@@ -84,10 +77,11 @@ class EventTrigger {
       virtual void readData ( tnstream& stream ) = 0;
       virtual void writeData ( tnstream& stream ) = 0;
       
-      virtual ASCString getName() const = 0;
+      virtual ASCString getDetailledName() const = 0;
+      virtual ASCString getTypeName() const = 0;
       virtual void setup() = 0;
       virtual void arm() {};
-      void setMap( tmap* gamemap_ ) { gamemap = gamemap_; };
+      void setMap( GameMap* gamemap_ ) { gamemap = gamemap_; };
       void setEvent( Event* ev ) { event = ev; };
       EventTriggerID getTriggerID() { return triggerID; };
 
@@ -99,27 +93,28 @@ class EventTrigger {
 class EventAction {
       EventActionID actionID;
    protected:
-      tmap* gamemap;
+      GameMap* gamemap;
       EventAction( EventActionID id ) : actionID ( id ), gamemap(NULL) {};
    public:
 
       virtual void readData ( tnstream& stream ) = 0;
       virtual void writeData ( tnstream& stream ) = 0;
-      virtual ASCString getName();
+      virtual ASCString getName() const = 0;
 
       virtual void execute( MapDisplayInterface* md ) = 0;
       virtual void setup() = 0;
-      void setMap( tmap* gamemap_ ) { gamemap = gamemap_; };
+      void setMap( GameMap* gamemap_ ) { gamemap = gamemap_; };
       EventActionID getActionID() { return actionID; };
       virtual ~EventAction() {};
 };
 
 class Event {
-      tmap& gamemap;
+      GameMap& gamemap;
 
       void clear();
    public:
-      Event( tmap& map_ );
+      Event( GameMap& map_ );
+      const GameMap* getMap() const { return &gamemap; };
 
       enum Status { Untriggered, Triggered, Timed, Executed } status;
 
@@ -159,37 +154,56 @@ class Event {
       virtual ~Event();
 };
 
-
-
-template < class AbstractProduct,
-           typename IdentifierType >
-class Factory{
+class VariableLocker {
+   bool& var;
    public:
-      typedef AbstractProduct* (*ObjectCreatorCallBack)();
-   private:
-      typedef map<IdentifierType, ObjectCreatorCallBack> CallbackMap;
-      CallbackMap callbackMap;
-   public:
-      bool registerClass( IdentifierType id, ObjectCreatorCallBack createFn ) { callbackMap[id] = createFn; return true; };
-      AbstractProduct* createObject( IdentifierType id )
-      {
-         typename CallbackMap::const_iterator i = callbackMap.find(id);
-         if ( i != callbackMap.end() )
-            return (i->second)();
-         else
-            fatalError("Factory: Object ID not found");
-		 return NULL;
-      };
+      VariableLocker( bool& variable ) : var ( variable ) { var = true; };
+      ~VariableLocker() { var = false; };
 };
 
-template<class Base, class Derived>
-Base* ObjectCreator()
-{
-   return new Derived;
-}
 
-typedef Loki::SingletonHolder< Factory< EventTrigger, EventTriggerID > > triggerFactory;
-typedef Loki::SingletonHolder< Factory< EventAction , EventActionID  > > actionFactory;
+template < class AbstractProduct, typename IdentifierType, typename ObjectCreatorCallBack = AbstractProduct*(*)(), typename NameType = ASCString >
+class FactoryWithNames : protected Factory<AbstractProduct, IdentifierType>
+{
+   private:
+      typedef std::map<NameType, IdentifierType> NameMap;
+      NameMap names;
+   public:
+      vector<NameType> getNames(){
+         vector<NameType> nameList;
+         
+         for ( typename NameMap::iterator i = names.begin(); i != names.end(); ++i )
+            nameList.push_back( i->first );
+         return nameList;
+      }
+
+      IdentifierType getID( const ASCString& name )
+      {
+         return names[name];
+      }
+
+      bool registerClass( IdentifierType id, typename FactoryWithNames<AbstractProduct, IdentifierType, ObjectCreatorCallBack, NameType>::ObjectCreatorCallBack createFn, Loki::Functor<NameType, TYPELIST_1(const IdentifierType&)> nameProvider )
+      {
+         return registerClass( id, createFn, nameProvider(id) );
+      }
+
+      bool registerClass( IdentifierType id, typename FactoryWithNames<AbstractProduct, IdentifierType, ObjectCreatorCallBack, NameType>::ObjectCreatorCallBack createFn, NameType name )
+      {
+         if ( Factory<AbstractProduct, IdentifierType>::registerClass ( id, createFn )) {
+            names[name] = id;
+            return true;
+         } else
+            return false;
+      }
+
+      AbstractProduct* createObject( IdentifierType id ) {
+         return Factory<AbstractProduct, IdentifierType>::createObject( id );
+      }
+
+};
+
+typedef Loki::SingletonHolder< FactoryWithNames< EventTrigger, EventTriggerID > > triggerFactory;
+typedef Loki::SingletonHolder< FactoryWithNames< EventAction , EventActionID  > > actionFactory;
 
 
 
