@@ -2,7 +2,7 @@
     \brief The old event system that is still required to load old maps
 */
 
-//     $Id: prehistoricevents.h,v 1.1 2004/01/16 19:14:55 mbickel Exp $
+//     $Id: prehistoricevents.h,v 1.3 2007-04-13 16:15:54 mbickel Exp $
 /*
     This file is part of Advanced Strategic Command; http://www.asc-hq.de
     Copyright (C) 1994-1999  Martin Bickel  and  Marc Schellenberger
@@ -29,6 +29,7 @@
 
 #include "typen.h"
 
-extern void  readOldEventLists ( pnstream stream, bool passedEvents, pmap spfld );
+class GameMap;
+extern void  readOldEventLists ( pnstream stream, bool passedEvents, GameMap* spfld );
 
 #endif
