@@ -60,37 +60,75 @@ void TechDependency::runTextIO ( PropertyContainer& pc )
 
 bool TechDependency::available( const Research& research ) const
 {
-   for ( vector<IntRange>::const_iterator j = blockingTechnologies.begin(); j != blockingTechnologies.end(); ++j )
+   return available( CheckTechAvailabilityFunctor( &research, &Research::techResearched )) == Available;
+}
+
+ResearchAvailabilityStatus TechDependency::available( CheckTechAvailabilityFunctor checkTechAvailability ) const
+{
+   for ( RequiredTechnologies::const_iterator j = blockingTechnologies.begin(); j != blockingTechnologies.end(); ++j )
        for ( int k = j->from; k <= j->to; ++k )
-          if ( research.techResearched ( k ))
-             return false;
+         if ( checkTechAvailability ( k ))
+             return NeverAvailable;
 
 
    if ( requiredTechnologies.size() ) {
-      for ( RequiredTechnologies::const_iterator j = blockingTechnologies.begin(); j != blockingTechnologies.end(); ++j )
-         for ( int k = j->from; k <= j->to; ++k )
-            if ( research.techResearched( k ))
-               return false;
-
       if ( requireAllListedTechnologies ) {
          for ( RequiredTechnologies::const_iterator j = requiredTechnologies.begin(); j != requiredTechnologies.end(); ++j )
             for ( int k = j->from; k <= j->to; ++k )
-               if ( !research.techResearched( k ))
-                  return false;
-          return true;
+               if ( !checkTechAvailability( k ))
+                  return UnavailableNow;
+         return Available;
       } else {
          for ( RequiredTechnologies::const_iterator j = requiredTechnologies.begin(); j != requiredTechnologies.end(); ++j )
             for ( int k = j->from; k <= j->to; ++k )
-                if ( research.techResearched ( k ))
-                  return true;
-         return false;
+               if ( checkTechAvailability ( k ))
+                  return Available;
+         return UnavailableNow;
       }
-   }
-
-   return true;
+   } else
+      return Available;
 }
 
 
+bool TechDependency::eventually_available_single( const Research& res, list<const Technology*>* dependencies, list<int>& stack, int id )
+{
+   Technology* tech = technologyRepository.getObject_byID( id );
+   if ( !tech )
+      return false;
+
+
+   
+   return tech->eventually_available( res, dependencies, stack );
+}
+
+bool TechDependency::eventually_available( const Research& res, list<const Technology*>* dependencies, list<int>& stack ) const
+{
+   if ( requireAllListedTechnologies ) {
+      for ( RequiredTechnologies::const_iterator j = requiredTechnologies.begin(); j != requiredTechnologies.end(); ++j )
+         for ( int k = j->from; k <= j->to; ++k ) 
+            if ( !eventually_available_single( res, dependencies, stack, k ))
+               return false;
+         
+         return true;
+   } else {
+      for ( RequiredTechnologies::const_iterator j = requiredTechnologies.begin(); j != requiredTechnologies.end(); ++j )
+         for ( int k = j->from; k <= j->to; ++k ) 
+            if ( eventually_available_single( res, dependencies, stack, k )) 
+               return true;
+            
+      return false;
+   }
+   return false;
+}
+
+bool TechDependency::eventually_available( const Research& res, list<const Technology*>* dependencies ) const
+{
+   list<int> stack;
+   return eventually_available( res, dependencies, stack );
+}
+
+
+
 int TechDependency::findInheritanceLevel( int id, vector<int>& stack, const ASCString& sourceTechName ) const
 {
    for ( RequiredTechnologies::const_iterator j = requiredTechnologies.begin(); j != requiredTechnologies.end(); ++j )
@@ -106,7 +144,7 @@ int TechDependency::findInheritanceLevel( int id, vector<int>& stack, const ASCS
                Technology* t = technologyRepository.getObject_byID( k );
                if ( t )
                   s += t->name + "\n";
-               longWarning ( s );
+               warning ( s );
                return -1;
             }
 
@@ -292,6 +330,52 @@ void TechDependency::writeInvertTreeOutput ( const ASCString techName, tnstream&
       }
 }
 
+ASCString TechDependency::showDebug( const Research& research ) const
+{
+   ASCString s;
+   if ( requireAllListedTechnologies )
+      s += "All of the following Technologies required:\n";
+   else
+      s += "One of the following Technologies required:\n";
+
+   s += "#indent=20,20#";
+   for ( RequiredTechnologies::const_iterator r = requiredTechnologies.begin(); r != requiredTechnologies.end(); ++r ) {
+      for ( int i = r->from; i <= r->to; ++i ) {
+         Technology* t = technologyRepository.getObject_byID(i);
+         if ( t ) {
+            s += "  " + t->name + " ";
+            if ( research.techResearched( t->id ))
+               s += "#fontcolor=0x00FF00#available";
+            else
+               s += "#fontcolor=0xFF0000#missing";
+            s += "#fontcolor=default#\n";
+         }
+      }
+   }
+   s += "#indent=0,0#";
+
+   if ( !blockingTechnologies.empty() ) {
+      s += "The following Technologies must NOT be researched:\n";
+      s += "#indent=20,20#";
+      for ( RequiredTechnologies::const_iterator r =  blockingTechnologies.begin(); r !=  blockingTechnologies.end(); ++r ) {
+         for ( int i = r->from; i <= r->to; ++i ) {
+            Technology* t = technologyRepository.getObject_byID(i);
+            if ( t ) {
+               s += "  " + t->name;
+               if ( !research.techResearched( t->id ))
+                  s += "#fontcolor=0x00FF00#not available";
+               else
+                  s += "#fontcolor=0xFF0000#available";
+               s += "#fontcolor=default#\n";
+            }
+         }
+      }
+   }
+
+   return s;
+}
+
+
 
 
 void TechAdapter::read ( tnstream& stream )
@@ -393,6 +477,25 @@ void TechAdapterDependency::writeInvertTreeOutput ( const ASCString& tech, tnstr
             (*i)->techDependency.writeInvertTreeOutput ( tech, stream, history, blockedPrintList, onlyWithBaseTechs );
 }
 
+ASCString TechAdapterDependency::showDebug( const Research& research ) const
+{
+   ASCString s;
+   if ( requireAllListedTechAdapter )
+      s += "All of the following TechAdapter required:\n";
+   else
+      s += "One of the following TechAdapter required:\n";
+
+   for ( RequiredTechAdapter::const_iterator r = requiredTechAdapter.begin(); r != requiredTechAdapter.end(); ++r ) {
+      s += "  " + *r;
+      if ( research.techAdapterAvail( *r ))
+         s += "#image=pad_symbol_ok.png#";
+      else
+         s += "#image=pad_symbol_no.png#";
+      s += "\n";
+   }
+
+   return s;
+}
 
 
 Technology::Technology()
@@ -404,7 +507,7 @@ Technology::Technology()
    techlevel = 0;
 }
 
-const int technologyVersion = 3;
+const int technologyVersion = 4;
 
 void Technology::read( tnstream& stream )
 {
@@ -426,6 +529,9 @@ void Technology::read( tnstream& stream )
    if ( version >= 3 )
       readClassContainer( blockingOtherTechnologies, stream );
 
+   if ( version >= 4 )
+      readClassContainer( secondaryIDs, stream );
+
 }
 
 void Technology::write( tnstream& stream ) const
@@ -440,6 +546,7 @@ void Technology::write( tnstream& stream ) const
    techDependency.write( stream );
    stream.writeInt ( relatedUnitID );
    writeClassContainer( blockingOtherTechnologies, stream );
+   writeClassContainer( secondaryIDs, stream );
 }
 
 void Technology::runTextIO ( PropertyContainer& pc )
@@ -448,6 +555,9 @@ void Technology::runTextIO ( PropertyContainer& pc )
    pc.addString( "Infotext", infotext);
 
    pc.addInteger( "Id", id );
+   if ( pc.find( "SecondaryIDs") || !pc.isReading())
+      pc.addIntegerArray("SecondaryIDs", secondaryIDs );
+
    pc.addInteger( "Researchpoints", researchpoints );
 
    pc.addInteger( "Techlevel", techlevel );
@@ -462,14 +572,50 @@ void Technology::runTextIO ( PropertyContainer& pc )
    pc.addInteger( "RelatedUnitID", relatedUnitID, 0 );
 }
 
+bool Technology::eventually_available( const Research& res, list<const Technology*>* dependencies ) const
+{
+   if ( res.techResearched( id ))
+      return false;
+   
+   list<int> stack;
+   return eventually_available( res, dependencies, stack );
+}
 
-Research :: Research ( )
+bool Technology::eventually_available( const Research& res, list<const Technology*>* dependencies, list<int>& stack ) const
+{
+   if ( res.techResearched( id ))
+      return true;
+   
+   if ( res.isBlocked( this ))
+      return false;
+
+   if ( find( stack.begin(), stack.end(), id ) != stack.end() )
+      return false;
+            
+   stack.push_back( id );
+   bool result = techDependency.eventually_available( res, dependencies, stack );
+   stack.pop_back();
+
+   if ( result ) {
+      if ( dependencies )
+         if ( find( dependencies->begin(), dependencies->end(), this) == dependencies->end() )
+            dependencies->push_back( this );
+      return true;
+   } else
+      return  false;
+      
+}
+
+
+
+Research :: Research ( )  : goal(NULL)
 {
    progress = 0;
    activetechnology = 0;
    ___loadActiveTech = 0;
    ___oldVersionLoader = false;
    techsAvail = true;
+   multiplier = 1;
 }
 
 
@@ -484,7 +630,7 @@ void Research :: read ( tnstream& stream )
 const int researchableWeaponImprovements = 8;
 
 
-const int researchVersion = -3;
+const int researchVersion = -5;
 
 void Research :: read_struct ( tnstream& stream, bool merge )
 {
@@ -535,6 +681,18 @@ void Research :: read_struct ( tnstream& stream, bool merge )
             readClassContainer( predefinedTechAdapter, stream );
          }
       }
+      if ( version <= -4 ) {
+         multiplier = stream.readInt();
+      }
+
+      if ( version <= -5 ) {
+         int g = stream.readInt();
+         if ( g > 0 )
+            goal = technologyRepository.getObject_byID( g );
+         else
+            goal = NULL;
+      } else
+         goal = NULL;
    }
 }
 
@@ -548,6 +706,12 @@ void Research :: write ( tnstream& stream ) {
 
    stream.writeInt ( techsAvail );
    writeClassContainer( predefinedTechAdapter, stream );
+   stream.writeInt( multiplier );
+   if ( goal )
+      stream.writeInt( goal->id );
+   else
+      stream.writeInt( 0 );
+            
 }
 
 
@@ -581,6 +745,15 @@ void Research :: read_techs ( tnstream& stream, bool merge )
    }
 }
 
+void Research::clear()
+{
+   progress = 0;
+   activetechnology = NULL;
+   goal = NULL;
+   developedTechnologies.clear();
+   predefinedTechAdapter.clear();
+   triggeredTechAdapter.clear();
+}
 
 bool Research::techResearched ( int id ) const
 {
@@ -610,39 +783,59 @@ void Research::evalTechAdapter()
 ASCString Research::listTriggeredTechAdapter() const
 {
    ASCString s;
-   for ( TriggeredTechAdapter::const_iterator i = triggeredTechAdapter.begin(); i != triggeredTechAdapter.end(); ++i )
+   for ( TriggeredTechAdapter::const_iterator i = triggeredTechAdapter.begin(); i != triggeredTechAdapter.end(); ++i ) {
+      s += "#fontsize=14# ";
       s += i->first + "\n";
+      s += "#fontsize=10#";
+      int counter = 0;
+      for ( TechAdapterContainer::iterator j = techAdapterContainer.begin(); j != techAdapterContainer.end(); ++j ) {
+         if ( (*j)->getName() == i->first ) {
+            if ( counter )
+               s += " ----\n";
+            ++counter;
+
+            s += (*j)->techDependency.showDebug( *this );
+         }
+      }
+      s += "#fontsize=14#\n";
+      
+
+   }
    
    return s;
 }
 
 
 
-Research::AvailabilityStatus Research::techAvailable ( const Technology* tech )
+
+bool Research::isBlocked( const Technology* tech ) const
+{
+   for ( int i = 0; i < developedTechnologies.size(); ++i ) {
+      Technology* t = technologyRepository.getObject_byID( developedTechnologies[i] );
+      if ( t && !t->techDependency.available ( *this ) ) {
+            // this is a root technology
+         for ( Technology::BlockingOtherTechnologies::iterator j = t->blockingOtherTechnologies.begin(); j != t->blockingOtherTechnologies.end(); ++j)
+            if ( j->from <= tech->id && tech->id <= j->to )
+               return true;
+      }
+   }
+   return false;
+
+}
+
+
+ResearchAvailabilityStatus Research::techAvailable ( const Technology* tech ) const
 {
    if ( techResearched( tech->id ))
-      return researched;
+      return Researched;
+
+   if ( isBlocked( tech ))
+      return NeverAvailable;
 
    if ( tech->techDependency.available( *this ) ) {
-      for ( int i = 0; i < developedTechnologies.size(); ++i ) {
-         Technology* t = technologyRepository.getObject_byID( developedTechnologies[i] );
-         if ( t && !t->techDependency.available ( *this ) ) {
-            // this is a root technology
-            for ( Technology::BlockingOtherTechnologies::iterator j = t->blockingOtherTechnologies.begin(); j != t->blockingOtherTechnologies.end(); ++j)
-                if ( j->from <= tech->id && tech->id <= j->to )
-                    return unavailable;
-         }
-      }
-
-      return available;
+      return Available;
    } else
-      return unavailable;
-
-/*
-    if ( tech->requireevent)
-       if ( ! actmap->eventpassed(tech->id, cenewtechnologyresearchable) )
-          fail = true;
-*/
+      return UnavailableNow;
 }
 
 
@@ -710,14 +903,39 @@ void Research::initchoosentechnology()
 }
 */
 
+int Research :: getResearchPerTurn() const
+{
+   int rppt = 0;
+   for ( Player::BuildingList::const_iterator i = map->player[player].buildingList.begin(); i != map->player[player].buildingList.end(); i++ )
+      rppt += (*i)->researchpoints;
+
+   for ( Player::VehicleList::const_iterator i = map->player[player].vehicleList.begin(); i != map->player[player].vehicleList.end(); i++ )
+      rppt += (*i)->researchpoints;
+   
+   return rppt * multiplier;
+}
+
+int Research :: currentTechAvailableIn() const
+{
+   if ( activetechnology ) {
+      int rpt = getResearchPerTurn();
+      if ( !rpt )
+         return 0;
+      else
+         return ( activetechnology->researchpoints - progress + rpt - 1) / rpt;
+   } else
+      return -1;
+}
+
+
 Research::~Research () {};
 
 
 
-Resources returnResourcenUseForResearch ( const pbuilding bld, int research )
+Resources returnResourcenUseForResearch ( const ContainerBase* bld, int research )
 {
    Resources res;
-   if ( bld->typ->nominalresearchpoints == 0 )
+   if ( bld->baseType->nominalresearchpoints == 0 )
       return res;
 
    int num = 0;
@@ -732,18 +950,24 @@ Resources returnResourcenUseForResearch ( const pbuilding bld, int research )
 
 
    for ( int r = 0; r < 3; ++r )
-      if ( bld->typ->maxplus.resource(r) < 0 )
-         if(  research > bld->typ->nominalresearchpoints ) {
-            float a = -bld->typ->maxplus.resource(r) / pow(double(bld->typ->nominalresearchpoints),2);
+      if ( bld->baseType->maxplus.resource(r) < 0 )
+         if(  research > bld->baseType->nominalresearchpoints ) {
+            float a = -bld->baseType->maxplus.resource(r) / pow(double(bld->baseType->nominalresearchpoints),2);
             res.resource(r) = int( pow(pow(double(research),2) * a, 0.98 + double(num)/50) );
          } else
-            res.resource(r) = -bld->typ->maxplus.resource(r) * research / bld->typ->nominalresearchpoints;
+            res.resource(r) = -bld->baseType->maxplus.resource(r) * research / bld->baseType->nominalresearchpoints;
 
    return res;
 }
 
+Resources returnResourcenUseForResearch ( const ContainerBase* bld )
+{
+   return returnResourcenUseForResearch ( bld, bld->researchpoints );
+}
+
+
 #if 0
-void returnresourcenuseforresearch ( const pbuilding bld, int research, int* energy, int* material )
+void returnresourcenuseforresearch ( const Building* bld, int research, int* energy, int* material )
 {
    /*
    double esteigung = 55;
