@@ -27,6 +27,8 @@
  #include <cstring>
  #include <map>
 
+ #include "libs/loki/Functor.h"
+
  #include "basestreaminterface.h"
  #include "basictypes.h"
  #include "typen.h"
@@ -34,17 +36,39 @@
 
 class Research;
 class Technology;
+class Building;
+class Vehicle;
+class Vehicletype;
+class GameMap;
+
+enum ResearchAvailabilityStatus { Researched, Available, UnavailableNow, NeverAvailable };
+
 
 class TechDependency: public LoadableItemType {
      typedef vector<IntRange> RequiredTechnologies;
+     //! if one of these technologies has been researched, this tech will be never be researchable. This allows exclusive technology branches
      RequiredTechnologies blockingTechnologies;
      
      RequiredTechnologies requiredTechnologies;
      bool         requireAllListedTechnologies;
-     //! if one of these technologies has been researched, this tech will be never be researchable. This allows exclusive technology branches
+     
+     static bool eventually_available_single( const Research& res, list<const Technology*>* dependencies, list<int>& stack, int id );
    public:
      TechDependency(){ requireAllListedTechnologies = true; };
+
      bool available( const Research& research ) const;
+     typedef Loki::Functor<bool, TYPELIST_1(int) > CheckTechAvailabilityFunctor;
+     ResearchAvailabilityStatus available( CheckTechAvailabilityFunctor checkTechAvailability ) const;
+
+     /** this will recursively scan through the tech tree to check if this item will be available sometime.
+     \param dependencies if not NULL, the technologies which have to be developed prior to this one will be added here
+     */
+     bool eventually_available( const Research& res, list<const Technology*>* dependencies ) const;
+     bool eventually_available( const Research& res, list<const Technology*>* dependencies, list<int>& stack ) const;
+
+
+     ASCString showDebug( const Research& research ) const;
+     
      void read ( tnstream& stream );
      void write ( tnstream& stream ) const;
      void runTextIO ( PropertyContainer& pc);
@@ -81,6 +105,7 @@ class TechAdapterDependency {
    public:
      TechAdapterDependency();
      bool available( const Research& research ) const;
+     ASCString showDebug( const Research& research ) const;
 
      void read ( tnstream& stream );
      void write ( tnstream& stream ) const;
@@ -96,8 +121,14 @@ class TechAdapterDependency {
 
      void*        icon;
      ASCString    infotext;
+
      int          id;
+     //! when loading a file and these IDs are encountered, this object will be used.
+     vector<int> secondaryIDs;
+
+
      int          researchpoints;
+
      ASCString    name;
      int          techlevel;
      int          relatedUnitID;
@@ -111,6 +142,9 @@ class TechAdapterDependency {
      //! if this Technology has been researched, it will not be possible to research the technologies given here. Only available for root technologies!
      BlockingOtherTechnologies blockingOtherTechnologies;
 
+     bool eventually_available( const Research& res, list<const Technology*>* dependencies ) const;
+     bool eventually_available( const Research& res, list<const Technology*>* dependencies, list<int>& stack ) const;
+     
 
      void read ( tnstream& stream );
      void write ( tnstream& stream ) const;
@@ -122,11 +156,14 @@ class TechAdapterDependency {
 
  class Research {
 
-     pmap map;
+     GameMap* map;
      int player;
 
      int ___loadActiveTech;
      bool ___oldVersionLoader;
+
+     int multiplier;
+     
    #ifdef karteneditor
    public:
    #endif   
@@ -144,11 +181,13 @@ class TechAdapterDependency {
 
      bool techResearched ( int id ) const;
 
-     enum AvailabilityStatus { researched, available, unavailable };
-
      int  progress;
+     //! the technology that is currently being researched
      const Technology* activetechnology;
 
+     //! the technology that is defined as long-term goal
+     const Technology* goal;
+
      bool vehicletypeavailable ( const Vehicletype* fztyp );
 
      void read ( tnstream& stream );
@@ -162,26 +201,38 @@ class TechAdapterDependency {
 
      void settechlevel ( int techlevel );
 
-     void chainToMap ( pmap _map, int _player ) { map = _map; player = _player; };
+     void chainToMap ( GameMap* _map, int _player ) { map = _map; player = _player; };
 
      //! Move the technology that is currently being reseached to the list of discovered technologies
      void addtechnology();
 
      void addanytechnology ( const Technology* tech );
 
-     AvailabilityStatus techAvailable ( const Technology* tech );
+     ResearchAvailabilityStatus techAvailable ( const Technology* tech ) const;
+
+     bool isBlocked( const Technology* tech ) const;
 
      /** is used by the chooseTechnology dialog: the first time no techs are available this variable is still true,
          so the dialog shows "now techs avail". THen it sets techAvail to false, preventing the same message at the
          beginning of each turn */
      bool techsAvail;
 
+     int getResearchPerTurn() const;
+     int currentTechAvailableIn() const;
+
+     void setMultiplier( int m ) { multiplier = m; };
+     int getMultiplier() { return multiplier; };
+     
      Research ( );
+     void clear();
      ~Research ();
  };
 
+ class ContainerBase;
+ 
 //! Calculates the resources that are needed to research the given number of research
-extern Resources returnResourcenUseForResearch ( const pbuilding bld, int research );
+extern Resources returnResourcenUseForResearch ( const ContainerBase* bld, int research );
+extern Resources returnResourcenUseForResearch ( const ContainerBase* bld );
 
 #endif
 
