@@ -23,6 +23,8 @@
 #include <cstring>
 #include <cerrno>
 #include <cstdio>
+#include <iostream>
+#include <fstream>
 
 #include <SDL_types.h>
 #include <SDL_endian.h>
@@ -55,11 +57,17 @@
 #define Sentinel    0xF0   /* the sentinel flag */
 #define BUFFER_SIZE 30000
 
-#define WriteCode(a,b,c) \
-            fprintf( outfile, "%c%c%c", a, b, c )
+#define WriteCode(a,b,c) fprintf( outfile, "%c%c%c", a, b, c )
 
 
 
+void fatalError ( const ASCString& s )
+{
+   fprintf( stderr, "%s \n", s.c_str() );
+   exit(1);
+}
+
+            
 int writeInt  ( FILE* f, int i )
 {
    i = SDL_SwapLE32(i);
@@ -80,10 +88,10 @@ const char* containerName = NULL;
 FILE *infile,
 *outfile;
 
-int rlemain ( char* argv1, char* argv2  )
+int rlemain ( const char* argv1, const char* argv2  )
 {
    unsigned char *buffer;
-   unsigned char prev_char;
+   unsigned char prev_char = 0 ;
    int  bytes_read,
    count,
    eof,
@@ -192,7 +200,7 @@ int rlemain ( char* argv1, char* argv2  )
 
 
 int
-bzmain( char *argv1, char* argv2  )
+bzmain( const char *argv1, const char* argv2  )
 {
    int size = filesize ( argv1 );
    void* buf = malloc ( size );
@@ -211,6 +219,35 @@ bzmain( char *argv1, char* argv2  )
 }
 
 
+ASCString getTempPath()
+{
+#ifdef _UNIX_
+  return "/tmp/";
+#else
+   ASCString name;
+   if ( getenv ( "temp" ))
+      name = getenv ( "temp" );
+   else
+      if ( getenv ( "tmp" ))
+         name = getenv ( "tmp" );
+      else {
+         name = pathdelimitterstring + ASCString("tmp");
+      }
+
+      appendbackslash( name );
+      return name;
+#endif
+
+}
+
+ASCString getTempFileName( int type )
+{
+   if ( type == 0 )
+      return getTempPath() + "temp.rle";
+   else
+      return getTempPath() + "temp.mzl";
+}
+
 
 
 dynamic_array<tcontainerindex> nindex;
@@ -251,7 +288,7 @@ void copyfile ( const char* name, const char* orgname, int size )
 char buf2[10000];
 
 /* mikem: seems to test a file for asc compression */
-void testcompress ( char* name, int size )
+void testcompress ( const char* name, int size )
 {
    if ( strcmp ( name, "temp.mzl") == 0
      || strcmp ( name, "temp.rle") == 0
@@ -261,29 +298,20 @@ void testcompress ( char* name, int size )
         return;
       }
       
-   char newname[1000];
-   char* orgname = name;
-   if ( patimat ( "*.pcx", name ) ) {
+   
+   ASCString newname;
+   bool deleteNewName = false;
+   
+   const char* orgname = name;
+   if ( patimat ( "*.pcx", name, true ) ) {
 
       FILE* infile = fopen ( name, filereadmode );
 
-      if ( getenv ( "temp" ))
-         strcpy ( newname, getenv ( "temp" ));
-      else
-         if ( getenv ( "tmp" ))
-            strcpy ( newname, getenv ( "tmp" ));
-         else {
-            strcpy ( newname, pathdelimitterstring );
-            strcat ( newname, "tmp" );
-         }
-      if ( strlen ( newname ) && newname[strlen ( newname ) -1] != pathdelimitter )
-         strcat ( newname, pathdelimitterstring );
+      newname = getTempPath() + name;
 
-      strcat ( newname, name );
-
-      FILE* outfile = fopen ( newname, filewritemode );
+      FILE* outfile = fopen ( newname.c_str(), filewritemode );
       if ( !outfile )
-         fatalError( "Unable to open " + ASCString(newname) + " for writing." );
+         fatalError( "Unable to open " + newname + " for writing." );
 
       int size = filesize ( name );
       char* pc = (char*) malloc ( size );
@@ -296,9 +324,10 @@ void testcompress ( char* name, int size )
       int w = fwrite ( pc, 1, size, outfile );
       fclose ( outfile );
       if ( w != size )
-         fatalError( "Could not write file " + ASCString(newname) );
+         fatalError( "Could not write file " + newname );
 
-      name = newname;
+      name = newname.c_str();
+      deleteNewName = true;
    }
 
    uncompsize += size;
@@ -329,20 +358,20 @@ void testcompress ( char* name, int size )
       int r;
       fflush ( stdout );
 
-      r = rlemain( name, "temp.rle");
+      r = rlemain( name, getTempFileName(0).c_str() );
       if ( r ) {
          fatalError ( "error executing rle1; errno is " + ASCString(strrr (errno)) );
          exit ( r );
       }
-      int rl = filesize ( "temp.rle" );
+      int rl = filesize ( getTempFileName(0).c_str() );
 
       fflush ( stdout );
-      r = bzmain ( name, "temp.mzl" );
+      r = bzmain ( name, getTempFileName(1).c_str() );
       if ( r ) {
          fatalError ( "error executing mbzip; errno is " + ASCString ( strrr ( errno)));
          exit ( r );
       }
-      int mz = filesize ( "temp.mzl" );
+      int mz = filesize ( getTempFileName(1).c_str() );
 
       if ( !size ) {
          fatalError(" encountered a file of size 0 !" );
@@ -360,16 +389,19 @@ void testcompress ( char* name, int size )
          if ( compr == rl ) {
             if ( verbose )
                printf ( "rle compressed" );
-            copyfile ( "temp.rle", orgname, compr );
+            copyfile ( getTempFileName(0).c_str(), orgname, compr );
          } else
             if ( compr == mz ) {
                if ( verbose )
                   printf ( "mzl compressed" );
-               copyfile ( "temp.mzl", orgname, compr );
+               copyfile ( getTempFileName(1).c_str(), orgname, compr );
             }
+
+      unlink(getTempFileName(0).c_str());
+      unlink(getTempFileName(1).c_str());
    }
-   if ( name == newname )
-      remove ( newname );
+   if ( deleteNewName )
+      remove ( newname.c_str() );
 
 }
 
@@ -379,15 +411,73 @@ void testcompress ( char* name, int size )
 
 
 
-
-/*
-int  readInt  ( void )
+void processFiles( const char* files, const char* orgWorkingDir )
 {
-   int i;
-   readdata2 ( i );
-   return SDL_SwapLE32( i );
+   int compress = 1;
+
+
+   char buf[10000];
+   strncpy( buf, files, 10000 );
+
+   char* filename = buf;
+   char* dirname = ".";
+
+   while ( strchr( buf, foreignPathDelimitter ))
+      *strchr( buf, foreignPathDelimitter ) = pathdelimitter;
+
+   if ( strchr( filename , pathdelimitter )) {
+      char* c = filename + strlen( filename );
+      while ( *c != pathdelimitter ) 
+         c -= 1;
+      
+      dirname = buf;
+      filename = c+1;
+      *c = 0;
+   }
+
+   
+
+   DIR * dirp = opendir( dirname );
+   if( dirp != NULL ) {
+      for(;;) {
+         struct ASC_direct *direntp = readdir( dirp );
+         if ( direntp == NULL )
+            break;
+
+         if ( patimat ( filename , direntp->d_name, true ) &&
+               strcmp ( direntp->d_name, "." ) != 0 &&
+               strcmp ( direntp->d_name, ".." ) != 0 ) {
+            int    fnd = 0;
+            int j   = 0;
+
+            while( !fnd && ( j < num ) ) {
+               //                for ( int j = 0; j < num; j++ )
+               if ( strcmpi ( nindex[j++].name, direntp->d_name ) == 0 )
+                  fnd = 1;
+            }
+
+            if ( !fnd ) {
+               if ( chdir(dirname)  != 0 )
+                  fatalError( "could not change directory (1)");
+
+               if ( verbose )
+                  printf( direntp->d_name );
+               if ( compress )
+                  testcompress ( direntp->d_name,  filesize(direntp->d_name) );
+               else {
+                  if ( verbose )
+                     printf ( " is not compressed, " );
+                  copyfile ( direntp->d_name, direntp->d_name,  filesize(direntp->d_name)  );
+               }
+               if ( chdir(orgWorkingDir)  != 0 )
+                  fatalError( "could not change directory (2)");
+            }
+         }
+      }
+      closedir( dirp );
+   }
+   
 }
-*/
 
 int main(int argc, char *argv[] )
 {
@@ -421,46 +511,26 @@ int main(int argc, char *argv[] )
    pos += fwrite ( containermagic, 1, 4, out );
    pos += writeInt( out, i );
 
+
+   char orgWorkingDir[10000];
+   if ( !getcwd( orgWorkingDir, 10000 ))
+      fatalError( "could not obtain working directory");
+
+
    for ( int df = cl.next_param(); df < argc-1; df++ ) {
-      int compress = 1;
-
-      DIR *dirp;
-      struct ASC_direct *direntp;
-
-      dirp = opendir( "." );
-      if( dirp != NULL ) {
-         for(;;) {
-            direntp = readdir( dirp );
-            if ( direntp == NULL )
-               break;
-
-            if ( patimat ( argv[df] , direntp->d_name ) &&
-                  strcmp ( direntp->d_name, "." ) != 0 &&
-                  strcmp ( direntp->d_name, ".." ) != 0 ) {
-               int    fnd = 0;
-               int j   = 0;
-
-               while( !fnd && ( j < num ) ) {
-                  //                for ( int j = 0; j < num; j++ )
-                  if ( strcmpi ( nindex[j++].name, direntp->d_name ) == 0 )
-                     fnd = 1;
-               }
-
-               if ( !fnd ) {
-                  if ( verbose )
-                     printf( direntp->d_name );
-                  if ( compress )
-                     testcompress ( direntp->d_name,  filesize(direntp->d_name) );
-                  else {
-                     if ( verbose )
-                        printf ( " is not compressed, " );
-                     copyfile ( direntp->d_name, direntp->d_name,  filesize(direntp->d_name)  );
-                  }
-               }
-            }
+      if ( argv[df][0] == '@' ) {
+         std::ifstream list( argv[df]+1 );
+         if ( !list )
+            fatalError( "could not open file " + ASCString(argv[df]) );
+         
+         while (! list.eof() ) {
+            ASCString filename;
+            getline (list, filename);
+            processFiles( filename.c_str(), orgWorkingDir  );
          }
-         closedir( dirp );
-      }
+      } else 
+         processFiles( argv[df], orgWorkingDir );
+      
    } /* endwhile */
 
    if ( verbose )
@@ -472,8 +542,12 @@ int main(int argc, char *argv[] )
       writeInt( out, nindex[i].name ? 1 : 0 );
       writeInt( out, nindex[i].start );
       writeInt( out, nindex[i].end );
-      if ( nindex[i].name )
+
+      if ( nindex[i].name ) {
+         for ( int j = 0; j < strlen ( nindex[i].name ); ++j )
+            nindex[i].name[j] = tolower( nindex[i].name[j] );
          fwrite ( nindex[i].name, 1, strlen ( nindex[i].name ) + 1, out );
+      }
 
    } /* endfor */
 
