@@ -1,5 +1,3 @@
-//     $Id: typen.h,v 1.157 2005/04/24 10:09:31 mbickel Exp $
-
 /*
      This file is part of Advanced Strategic Command; http://www.asc-hq.de
      Copyright (C) 1994-1999  Martin Bickel  and  Marc Schellenberger
@@ -26,10 +24,12 @@
 #include <time.h>
 #include <list>
 #include <bitset>
+#include <map>
 #include <SDL_types.h>
 
 #include "global.h"
 
+
 #ifdef HAVE_LIMITS
  #include <limits>
 #else
@@ -38,32 +38,15 @@
 
 #include "basictypes.h"
 
-// #include <values.h>
-
-#include "pointers.h"
-#include "tpascal.inc"
-#include "misc.h"
 #include "basestrm.h"
-#include "errors.h"
-
-// #include "password.h"
+#include "messaginghub.h"
 
 
+//! the color depth of the gamemap
+const int gamemapPixelSize = 4;
 
-
-
-//! A Ellipse that is used for highlighting elements of the screen during the tutorial
-class  EllipseOnScreen {
-   public:
-      int x1, y1, x2, y2;
-      int color;
-      float precision;
-      int active;
-      EllipseOnScreen ( void ) { active = 0; };
-      void paint ( void );
-      void read( tnstream& stream );
-      void write ( tnstream& stream );
-   };
+//! the number of players that ASC can manage. This includes the neutral semi-played, which can't do anything
+const int playerNum = 9;
 
 
 //! The number of different weapon types
@@ -75,7 +58,7 @@ const int cwettertypennum = 6;
 //! The number of vehicle categories; Each category has its own move malus
 const int cmovemalitypenum  = 18;
 
-//! The movemalus type for a building. It is used for #SingleWeapon.targets_not_hittable .
+//! The movemalus type for a building. It is used for #SingleWeapon::targetingAccuracy .
 const int cmm_building = 11;
 const int cmm_trooper = 7;
 
@@ -94,6 +77,8 @@ const int resourceTypeNum = 3;
 //! The number of different resources that ASC uses
 const int resourceNum = resourceTypeNum;
 
+class PropertyContainer;
+
 //! The Container for the three different Resources that ASC uses.
 class Resources {
   public:
@@ -135,6 +120,10 @@ class Resources {
      void runTextIO ( PropertyContainer& pc, const Resources& defaultValue );
      static const char* name( int r );
      ASCString toString();
+     
+     static const int materialColor = 0xff0000; // 0x8b3d1e;
+     static const int fuelColor = 0x00ff00; // 0x8b7800;
+     static const int energyColor = 0x3b2dd7;
 };
 
 extern Resources operator- ( const Resources& res1, const Resources& res2 );
@@ -156,26 +145,18 @@ class ResourceMatrix {
 
 
 
+
 //! the time in ASC, measured in turns and moves
 struct GameTime {
   GameTime() { abstime = 0; };
-  int move() { return abstime % 0x10000; };
-  int turn() { return abstime / 0x10000; };
+  int move() const { return abstime % 0x10000; };
+  int turn() const { return abstime / 0x10000; };
   static bool comp ( const GameTime& a, const GameTime& b ) { return a.abstime > b.abstime; };
   void set ( int turn, int move ) { abstime = (turn * 0x10000) + move ; };
   int abstime;
 };
 
 
-//! the image for a terraintype ( #tterraintype ) that is shown on the small map
-struct FieldQuickView {
-      char p1;
-      char p3[3][3];
-      char p5[5][5];
-};
-
-
-
 //! Coordinate on the twodimensional map
 class MapCoordinate {
          public:
@@ -185,6 +166,7 @@ class MapCoordinate {
             MapCoordinate ( int _x, int _y) : x(_x), y(_y) {};
             bool operator< ( const MapCoordinate& mc ) const { return y < mc.y || ( y == mc.y && x < mc.x );};
             bool operator== ( const MapCoordinate& mc ) const { return y == mc.y && x == mc.x;};
+            bool operator!= ( const MapCoordinate& mc ) const { return y != mc.y || x != mc.x;};
             void write( tnstream& stream ) const { stream.writeInt ( 3000 ); stream.writeInt ( x ); stream.writeInt ( y); };
             void read( tnstream& stream ) {
                int vers = stream.readInt ( );
@@ -195,8 +177,20 @@ class MapCoordinate {
                y = stream.readInt ( );
             };
             bool valid() const { return x >= 0 && y >= 0 ; } ;
+	         void move(int width, int height);
+            ASCString toString() const;
       };
 
+inline MapCoordinate operator-( const MapCoordinate& a, const MapCoordinate& b )
+{
+  return MapCoordinate(a.x - b.x, a.y - b.y );
+}  
+      
+inline MapCoordinate operator+( const MapCoordinate& a, const MapCoordinate& b )
+{
+   return MapCoordinate(a.x + b.x, a.y + b.y );
+}
+      
 
 //! Coordinate on the map including height
 class MapCoordinate3D : public MapCoordinate {
@@ -204,6 +198,7 @@ class MapCoordinate3D : public MapCoordinate {
          public:
             int getBitmappedHeight ( ) const { if ( z >= 0 ) return 1<<z; else return 0;};
             int getNumericalHeight ( ) const { return z; };
+            void setNumericalHeight ( int nh ) { z = nh; };
             // MapCoordinate3D& operator= ( const MapCoordinate& mc ) { x = mc.x; y = mc.y; z = -1 );
             MapCoordinate3D ( ) : MapCoordinate(), z(-1) {};
             MapCoordinate3D ( int _x, int _y, int bitmappedz) : MapCoordinate ( _x, _y ), z ( log2(bitmappedz) ) {};
@@ -237,11 +232,32 @@ class LoadableItemType {
        virtual ~LoadableItemType() {};
 };
 
+
+template< typename T> 
+class deallocating_vector : public vector<T> {
+   public:
+      ~deallocating_vector() {
+         for ( typename vector<T>::iterator i = vector<T>::begin(); i != vector<T>::end(); ++i )
+            delete *i;
+      };      
+};
+
+template< typename T, typename U>
+class deallocating_map : public std::map<T,U> {
+   public:
+      ~deallocating_map() {
+         for ( typename std::map<T,U>::iterator i = std::map<T,U>::begin(); i != std::map<T,U>::end(); ++i )
+            delete i->second;
+      };
+};
+
+
 class IntRange {
      public:
            int from;
            int to;
            IntRange(): from(-1), to(-1) {};
+           IntRange( int oneValue ): from(oneValue), to(oneValue) {};
            IntRange( int from_, int to_ ): from(from_), to(to_) {};
            void read ( tnstream& stream );
            void write ( tnstream& stream ) const;
@@ -250,123 +266,6 @@ class IntRange {
 extern vector<IntRange> String2IntRangeVector( const ASCString& t );
 
 
-template<typename C>
-void writePointerContainer ( const C& c, tnstream& stream  )
-{
-   stream.writeInt ( 1 );
-   stream.writeInt ( c.size() );
-   typedef typename C::const_iterator IT;
-   for ( IT i = c.begin(); i != c.end(); ++i )
-      (*i)->write ( stream );
-}
-
-template<typename BaseType>
-void readPointerContainer ( vector<BaseType*>& v, tnstream& stream  )
-{
-   stream.readInt(); // version
-   int num = stream.readInt();
-   v.clear();
-   for ( int i = 0; i < num; ++i ) {
-      BaseType* bt = new BaseType;
-      bt->read( stream );
-      v.push_back( bt );
-   }
-}
-
-
-
-template<typename C>
-void writeClassContainer ( const C& c, tnstream& stream  )
-{
-   stream.writeInt ( 1 );
-   stream.writeInt ( c.size() );
-   typedef typename C::const_iterator IT;
-   for ( IT i = c.begin(); i != c.end(); ++i )
-      i->write ( stream );
-}
-
-template<typename C>
-void readClassContainer ( C& c, tnstream& stream  )
-{
-   int version = stream.readInt();
-   int num = stream.readInt();
-   c.clear();
-   for ( int i = 0; i < num; ++i ) {
-      typedef typename C::value_type VT;
-      VT vt;
-      vt.read( stream );
-      c.push_back( vt );
-   }
-}
-
-template<>
-inline void writeClassContainer<> ( const vector<ASCString>& c, tnstream& stream  )
-{
-   stream.writeInt ( 1 );
-   stream.writeInt ( c.size() );
-   typedef vector<ASCString>::const_iterator IT;
-   for ( IT i = c.begin(); i != c.end(); ++i )
-      stream.writeString(*i);
-}
-
-
-template<>
-inline void readClassContainer<> ( vector<ASCString>& c, tnstream& stream  )
-{
-   stream.readInt(); // version
-   int num = stream.readInt();
-   c.clear();
-   for ( int i = 0; i < num; ++i )
-      c.push_back( stream.readString() );
-}
-
-template<>
-inline void writeClassContainer<> ( const vector<int>& c, tnstream& stream  )
-{
-   stream.writeInt ( 1 );
-   stream.writeInt ( c.size() );
-   typedef vector<int>::const_iterator IT;
-   for ( IT i = c.begin(); i != c.end(); ++i )
-      stream.writeInt(*i);
-}
-
-
-template<>
-inline void readClassContainer<> ( vector<int>& c, tnstream& stream  )
-{
-   stream.readInt(); // version
-   int num = stream.readInt();
-   c.clear();
-   for ( int i = 0; i < num; ++i )
-      c.push_back ( stream.readInt() );
-}
-
-template<>
-inline void writeClassContainer<> ( const vector<pair<int,int> >& c, tnstream& stream  )
-{
-   stream.writeInt ( 1 );
-   stream.writeInt ( c.size() );
-   typedef vector<pair<int,int> >::const_iterator IT;
-   for ( IT i = c.begin(); i != c.end(); ++i ) {
-      stream.writeInt(i->first);
-      stream.writeInt(i->second );
-   }
-}
-
-
-template<>
-inline void readClassContainer<> ( vector<pair<int,int> >& c, tnstream& stream  )
-{
-   stream.readInt(); // version
-   int num = stream.readInt();
-   c.clear();
-   for ( int i = 0; i < num; ++i ) {
-       int first = stream.readInt();
-       int second = stream.readInt();
-       c.push_back ( make_pair(first,second) );
-   }
-}
-
 
 
 class MoveMalusType {
@@ -393,173 +292,6 @@ class MoveMalusType {
 
 
 
-/////////////////////////////////////////////////////////////////////
-/////////////////////////////////////////////////////////////////////
-/// Even more miscellaneous structures...
-/////////////////////////////////////////////////////////////////////
-/////////////////////////////////////////////////////////////////////
-
-#pragma pack(1)
-
-typedef struct tguiicon* pguiicon ;
-struct tguiicon {
-  void*      picture[2];
-  char         txt[31];
-  unsigned char         id;
-  char         key[4];
-  Uint16         realkey[4];
-  unsigned char         order;
-};
-
-
-struct ticons {
-   struct {
-     void      *pfeil1, *pfeil2;
-   } weapinfo;
-   void*        statarmy[3];
-   void*        height[8];      // fuer vehicleinfo - DLG-Box
-   void*        height2[3][8];  // fuer vehicleinfo am map
-   void*        player[8];      // aktueller Spieler in der dashboard: FARBE.RAW
-   void*        allianz[8][3];  // Allianzen in der dashboard: ALLIANC.RAW 
-   void*        diplomaticstatus[8]; 
-   void*        selectweapongui[13];
-   void*        selectweaponguicancel;
-   void*        unitinfoguiweapons[14];
-   void*        experience[maxunitexperience+1];
-   void*        wind[9];
-   void*        windarrow;
-   void*        stellplatz;
-   void*        guiknopf;   // reingedr?ckter knopf
-   void*        computer;
-   void*        windbackground;
-   void*        smallmapbackground;
-   void*        weaponinfo[5];
-   void*        X;
-   struct {
-     struct       {
-         void* active;
-         void* inactive;
-         void* repairactive;
-         void* repairinactive;
-         void* movein_active;
-         void* movein_inactive;
-     } mark;
-     struct       {
-         struct {
-           void* start;
-           void* active;
-           void* inactive;
-         } netcontrol;
-         struct {
-           void* start;
-           void* button;
-           void* buttonpressed;
-           void* schieber[4];
-           void* schiene;
-         } ammoproduction;
-         struct {
-           void* start;
-         } resourceinfo;
-         struct {
-           void* start;
-         } windpower;
-         struct {
-           void* start;
-         } solarpower;
-         struct {
-           void* start;
-           void* button;
-           void* buttonpressed;
-           void* schieber[4];
-           void* schiene;
-           void* schieneinactive;
-           void* singlepage[2];
-           void* plus[2];
-           void* minus[2];
-         } ammotransfer;
-         struct {
-           void* start;
-           void* button[2];
-           void* schieber;
-         } research;
-         struct {
-           void* start;
-           // void* button[2];
-           void* schieber;
-         } conventionelpowerplant;
-         struct {
-           void* start;
-           void* height1[8];
-           void* height2[8];
-           void* repair;
-           void* repairpressed;
-           void* block;
-         } buildinginfo;
-         struct {
-           void* start;
-                  void* zeiger;
-           void* button[2];
-           void* resource[2];
-           void* graph;
-           void* axis[3];
-           void* pageturn[2];
-         
-           void* schieber;
-         } miningstation;
-         struct {
-           void* start;
-                  void* zeiger;
-           void* schieber;
-         } mineralresources;
-         struct {
-           void* start;
-           void* height1[8];
-           void* height2[8];
-           void* sum;
-         } transportinfo;
-     } subwin;
-     union {
-        void* sym[11][2];
-        struct {
-          void*  ammotransfer[2];
-          void*  research[2];
-          void*  resourceinfo[2];
-          void*  netcontrol[2];
-          void*  solar[2];
-          void*  ammoproduction[2];
-          void*  wind[2];
-          void*  powerplant[2];
-          void*  buildinginfo[2];
-          void*  miningstation[2];
-          void*  transportinfo[2];
-          void*  mineralresources[2];
-        } a;
-     } lasche;
-
-     void* tabmark[2];
-     void* container_window;
-   } container;
-   struct {
-     void* bkgr;
-     void* orgbkgr;
-   } attack;
-   void*        pfeil2[8];     // beispielsweise fuer das Mouse-Scrolling 
-   void*        mousepointer;
-   void*        fieldshape;
-   void*        hex2octmask;
-   void*        mapbackground;
-   void*        mine[8]; // explosive mines
-   struct {                  
-               void*     nv8;
-               void*     va8;
-               void*     fog8;
-    } view;  
-};
-
-
-
-
-#pragma pack()
 
 /////////////////////////////////////////////////////////////////////
 /////////////////////////////////////////////////////////////////////
@@ -589,38 +321,6 @@ extern  const char*  choehenstufen[8] ;
  #define chsatellit 128  
 
 
-#define cwaffentypennum 13
- extern const char*  cwaffentypen[cwaffentypennum] ;
- #define cwcruisemissile 0
- #define cwcruisemissileb ( 1 << cwcruisemissile )
- #define cwminen 1
- #define cwmineb ( 1 << cwminen   )
- #define cwbombn 2
- #define cwbombb ( 1 << cwbombn  )
- #define cwairmissilen 3
- #define cwairmissileb ( 1 << cwairmissilen  )
- #define cwgroundmissilen 4
- #define cwgroundmissileb ( 1 << cwgroundmissilen  )
- #define cwtorpedon 5
- #define cwtorpedob ( 1 << cwtorpedon  )
- #define cwmachinegunn 6
- #define cwmachinegunb ( 1 << cwmachinegunn )
- #define cwcannonn 7
- #define cwcannonb ( 1 << cwcannonn )
- #define cwweapon ( cwcruisemissileb | cwbombb | cwairmissileb | cwgroundmissileb | cwtorpedob | cwmachinegunb | cwcannonb | cwlaserb )
- #define cwshootablen 11
- #define cwshootableb ( 1 << cwshootablen  )
- #define cwlasern 10
- #define cwlaserb ( 1 << cwlasern  )
- #define cwammunitionn 9
- #define cwammunitionb ( 1 << cwammunitionn )
- #define cwservicen 8
- #define cwserviceb ( 1 << cwservicen )
- #define cwobjectplacementn 12
- #define cwobjectplacementb ( 1 << cwobjectplacementn )
- extern const int cwaffenproduktionskosten[cwaffentypennum][3];  /*  Angabe: Waffentyp; energy - Material - Sprit ; jeweils fuer 5er Pack */
-
-
 
 
 
@@ -631,7 +331,7 @@ extern const char*  resourceNames[3];
 
 
 extern const char*  cmovemalitypes[cmovemalitypenum];
-
+extern const char*  moveMaliTypeIcons[cmovemalitypenum];
 
 const int experienceDecreaseDamageBoundaryNum = 4;
 extern const int experienceDecreaseDamageBoundaries[experienceDecreaseDamageBoundaryNum];
@@ -643,11 +343,8 @@ extern const int experienceDecreaseDamageBoundaries[experienceDecreaseDamageBoun
 /////////////////////////////////////////////////////////////////////
 /////////////////////////////////////////////////////////////////////
 
-#define guiiconsizex 49  
-#define guiiconsizey 35  
 
-
- #define maxmalq 10  
+ #define maxmalq 10
  #define minmalq 10
  #define fieldxsize 48    /*  Breite eines terrainbildes  */ 
  #define fieldysize 48  
@@ -696,6 +393,7 @@ extern const int experienceDecreaseDamageBoundaries[experienceDecreaseDamageBoun
 
 
 
+
 /////////////////////////////////////////////////////////////////////
 /////////////////////////////////////////////////////////////////////
 /// Constants that define the behaviour of units and buildings
@@ -714,24 +412,11 @@ const int submarineMovement = 11;
 
 #define mineputmovedecrease 10
 #define mineremovemovedecrease 10
-#define streetmovemalus 8
-#define railroadmovemalus 8
-#define searchforresorcesmovedecrease 8
-
 
 #define fusstruppenplattfahrgewichtsfaktor 2  
 #define mingebaeudeeroberungsbeschaedigung 80  
 
-#define autorepairdamagedecrease 10    // only for old units ; new one use autorepairrate
-
-#define weaponpackagesize 1
-
-#define brigde1buildcostincrease 12       // jeweils Basis 8; flaches Wasser
-#define brigde2buildcostincrease 16       // jeweils Basis 8; mitteltiefes Wasser
-#define brigde3buildcostincrease 36       // jeweils Basis 8; tiefes Wasser
-
-
-#define lookintoenemytransports false  
+#define lookintoenemytransports false
 #define lookintoenemybuildings false
 
 #define recyclingoutput 2    /*  Material div RecyclingOutput  */
@@ -745,10 +430,6 @@ const int maxwindspeed = 60;          // Wind with a strength of 255 means that
 
 #define mine_movemalus_increase 50   // percent
 
-#define tfieldtemp2max 255
-#define tfieldtemp2min 0
-
-
 #define cnet_storeenergy        0x001           // es wird garantiert,  dass material immer das 2 und fuel das 4 fache von energy ist
 #define cnet_storematerial      0x002
 #define cnet_storefuel          0x004
@@ -766,8 +447,8 @@ const int maxwindspeed = 60;          // Wind with a strength of 255 means that
 #define cnet_stopfueloutput     0x800
 
 
-#define resource_fuel_factor 100         // die im boden liegenden Bodenschtzen ergeben effektiv soviel mal mehr ( bei Bergwerkseffizienz 1024 )
-#define resource_material_factor 100     // "
+#define resource_fuel_factor 80         // die im boden liegenden Bodenschtzen ergeben effektiv soviel mal mehr ( bei Bergwerkseffizienz 1024 )
+#define resource_material_factor 80     // "
 
 #define destruct_building_material_get 2 // beim Abreissen erhlt man 1/2 des eingesetzten Materials zur?ck
 #define destruct_building_fuel_usage 10  // beim Abreissen wird 10 * fuelconsumption Fuel fuelconsumptiont
@@ -781,12 +462,10 @@ const int maxwindspeed = 60;          // Wind with a strength of 255 means that
 //! The maximum number of fields a mining station can extract mineral resources from; Counted from its entry
 const int maxminingrange = 10;
 
-const float productionLineConstructionCostFactor = 0.5;
-const float productionLineRemovalCostFactor = 0.2;
+const double productionLineConstructionCostFactor = 0.5;
+const double productionLineRemovalCostFactor = 0.2;
 
 
-#define objectbuildmovecost 16  // vehicle->movement -= (8 + ( fld->movemalus[0] - 8 ) / ( objectbuildmovecost / 8 ) ) * kosten des obj
-
 
 extern const int csolarkraftwerkleistung[];
 
@@ -798,9 +477,6 @@ extern const char* cnetcontrol[cnetcontrolnum];
 
 extern const char* cgeneralnetcontrol[];
 
-#define unspecified_error 9999
-
-#define greenbackgroundcol 156
-
+const int maxViewRange = 255;
 
 #endif
